<?php
/*
Plugin Name: Clean Options
Plugin URI: http://www.mittineague.com/dev/co.php
Description: finds orphaned options and allows for their removal from the wp_options table
Version: 1.1.1
Author: Mittineague
Author URI: http://www.mittineague.com
*/

/*
* Change Log
* 
* 1.1.1 21-Feb-2009
* - query syntax change
* - query error catching added
* - error scope changes
*
* 1.1.0 RC 27-Jan-2009
* - limit 'delete all' rss delete to < 100 highest id
* - added rss_hash limited find
* - fixed and updated the $known_ok array
* - improved robustness
* - - fixed searchdir() return type initialization
* - - set explicit return type in $wpdb->get_results queries
* - - ini_get('safe_mode') fixes
* - changed found rss_hash options section
* - added javascript select/deselect all
* - various other minor tweaks
* 
* 1.0.0 RC 12-Nov-2008
* - increased memory limit from 32M to 64M
* - added remove all rss_hash section
* 
* Beta 0.9.7 06-Aug-2008
* - provided for time limit increase
* - updated the $known_ok array (for WordPress 2.6)
* - added test for empty option_name field [autoload != yes block]
* 
* Beta 0.9.6 19-Oct-2007
* - added test for empty option_name field [autoload = yes block]
* - tweaked error handling
* 
* Beta 0.9.5 18-Oct-2007
* - updated the $known_ok array
* - scoped $cur_wp_ver
* - removed $wpdb->hide_errors() from get_all_no_autoload_options()
* - changed WP_Error obj syntax
* - removed global $wp_queries from get_all_no_autoload_options()
* - replaced get_alloptions() with get_all_yes_autoload_options()
* 
* Beta 0.9.4 06-Oct-2007
* - added WordPress ver. 2.3 compatibility
* - tweaked error handling
* 
* Beta 0.9.3 06-Jul-2007
* - updated/improved WP core options array
* - provided for memory limit increase
* - optimized memory usage
* 
* Beta 0.9.2 25-Apr-2007
* - improved protection against accidental removal of WP core options
* - - expanded the $known_ok array
* - - test for non-default install prefix user_roles option
* - - added backup suggestion
* 
* Beta 0.9.1 24-Apr-2007
* - changed a 'hard-coded' wp_ to $wpdb->
* 
* Beta 0.9.0 22-Apr-2007
* - added get_all "rss_" options
* - changed str_replace() to wordwrap()
* 
* Beta 0.7.1 17-Apr-2007
* - added "Further Information" section
* 
* Beta 0.7.0 13-Apr-2007
*/

/*
/--------------------------------------------------------------------\
|                                                                    |
| License: GPL                                                       |
|                                                                    |
| Clean Options Plugin - allows removal of orphaned options          |
| Copyright (C) 2007, Mittineague, www.mittineague.com               |
| All rights reserved.                                               |
|                                                                    |
| This program is free software; you can redistribute it and/or      |
| modify it under the terms of the GNU General Public License        |
| as published by the Free Software Foundation; either version 2     |
| of the License, or (at your option) any later version.             |
|                                                                    |
| This program is distributed in the hope that it will be useful,    |
| but WITHOUT ANY WARRANTY; without even the implied warranty of     |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      |
| GNU General Public License for more details.                       |
|                                                                    |
| You should have received a copy of the GNU General Public License  |
| along with this program; if not, write to the                      |
| Free Software Foundation, Inc.                                     |
| 51 Franklin Street, Fifth Floor                                    |
| Boston, MA  02110-1301, USA                                        |   
|                                                                    |
\--------------------------------------------------------------------/
*/

function mitt_add_co_page()
{
	if ( function_exists('add_management_page') )
	{
		add_management_page('Clean Options', 'CleanOptions', 8, basename(__FILE__), 'mitt_co_page');
	}
	$cononce = md5('cleanoptions');
}

function mitt_co_css()
{
?>
<style type="text/css">
div.submit {
  margin-top: -1.2em;
 }
#or_table {
  border: 1px solid #000;
  margin-bottom: 1em;
 }
#or_table th {
  border-bottom: 1px solid #777;
 }
#or_table td {
  padding: .1em;
  border-right: 1px solid #bbb;
  border-bottom: 1px solid #bbb;
 }
div.errors {
  border: solid #f00 2px;
  margin-top: 1em;
  padding: 1em;
 }
div.errors b {
  color: #f00;
 }
</style>
<?php
}

function mitt_co_page()
{
	global $cononce, $co_err_obj;

	define('CO_MIN_BEFORE_SHOW', 500);	// arbitrary limit - 500
	define('CO_BATCH_SIZE', 300);	// arbitrary size - 300
	define('CO_KEEP_NEWER', 100);	// arbitrary offset - 100

/* hopefully 64M is enough memory, (16M should be plenty)
* if memory limit is not at least 64M, ini_set() it
* 16M = 16777216 bytes
* 32M = 33554432 bytes
* 64M = 67108864 bytes
*/
	$co_mem_lim = ini_get('memory_limit');

	function co_return_bytes($val)
	{
		$val = trim($val);
		$last = strtolower( $val { strlen($val)-1 } );
		switch($last)
		{
			case 'g':
				$val *= 1024;
			case 'm':
				$val *= 1024;
			case 'k':
				$val *= 1024;
			default:
				$val = 0;
		}
		return $val;
	}

	$mem_lim_as_bytes = co_return_bytes($co_mem_lim);

	function get_opt_tbl_len()
	{
		global $wpdb;
		$get_tbl_len = $wpdb->get_var("SELECT COUNT(option_id) FROM $wpdb->options");
		return $get_tbl_len;
	}

	$opt_tbl_len = get_opt_tbl_len();

	if ( ($opt_tbl_len > CO_BATCH_SIZE) && ($mem_lim_as_bytes < 67108864) && !ini_get('safe_mode') )
	{
		ini_set('memory_limit', '64M');
	}

	if ( ($opt_tbl_len > CO_BATCH_SIZE) && !ini_get('safe_mode') )
	{
		$co_time_lim = ini_get('max_execution_time');
		if ( $co_time_lim < 60 )
		{
			set_time_limit(60);
		}
	}

	$cur_wp_ver = get_bloginfo('version');
	$cur_wp_ver = substr( $cur_wp_ver, 0, 3 );

	$common_form_message = '<p>Listed Options are those that are found in the wp_options table but are not referenced by "get_option" or "get_settings" by any of the PHP files located within your blog directory. If you have deactivated plugins and / or non-used themes in your directory, the associated options will not be considered orphaned until the files are removed.</p>';

	$del_all_warning = '<p><b>!WARNING!</b> To avoid excessive memory use, using "Delete ALL &#39;rss&#39; Options" does not attempt to list all of the "rss_hash" options, nor will you be able to preview the contents.<br />"Delete ALL &#39;rss&#39; Options" attempts to provide some measure against the deletion of current "rss_hash" options by not deleting any "rss_hash" rows with an option_id newer than (the last row id - ' . CO_KEEP_NEWER . '). Because plugins and themes also add rows to the wp-optons table, depending on your recent installation history, this may remove <b>ALL</b> of the "rss_hash" options, both older AND <b>CURRENT</b> ones, no questions asked.<br />Although removing current "rss_hash" rows should not "break" your WordPress blog (they should be entered again next time the feed is cached), please <b>BACKUP</b> your database <b>BEFORE</b> doing this.<br />After using "Delete ALL &#39;rss&#39; Options", you should "Find Orphaned Options" to clean the wp_options table further.</p><p>It is <b>highly recommended</b> that you Find only the ' . CO_BATCH_SIZE . ' oldest existing rss_hash options Instead and repeat the process until the number becomes manageable.';

	function add_wp_error($code, $message)
	{
		global $co_err_obj;

		if ( !is_object($co_err_obj) )
		{
			$co_err_obj = new WP_Error();
		}

		if ( is_object ($co_err_obj) )
		{
			$co_err_obj->add($code, $message);
		}
		else
		{
			echo '<div class="errors"><b>WARNING !!</b><br />';
			echo $code . ': ' . $message . '</div>';
		}
	}

	function check_for_wp_errors()
	{
		global $co_err_obj;

		if ( !empty($co_err_obj) )
		{
			echo '<div class="errors">';
			echo "<b>WARNING !!</b><dl>";
			foreach ($co_err_obj->get_error_codes() as $err_code)
			{
				foreach ($co_err_obj->get_error_messages($err_code) as $err_msg)
				{
					echo '<dt>' . $err_code . '</dt>';
					echo '<dd>' . $err_msg . '</dd>';
				}
			}
			echo '</dl></div>';
		}



	}

/* Find Orphans Section */

	if ( isset($_POST['find_orphans']) )
	{
		check_admin_referer('clean-options-find-orphans_' . $cononce);
?>
		<div class="wrap">
		<h2>Clean Options</h2>
		<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
		function searchdir($dir)
		{
			$file_list = array();
			$stack[] = $dir;
			while ($stack)
			{
				$current_dir = array_pop($stack);
				if ( $dh = opendir($current_dir) )
				{
					while ( ($file = readdir($dh)) !== false )
					{
						if ( ($file !== '.') && ($file !== '..') )
						{
							$current_file = "{$current_dir}/{$file}";
							$get_ext = pathinfo($current_file);
							if ( is_file($current_file) && ($get_ext['extension'] == 'php') )
							{
								$file_list[] = $current_file;
							}
							elseif ( is_dir($current_file) )
							{
								$stack[] = $current_file;
							}
						}
					}
					closedir($dh);
				}
				else
				{
					add_wp_error('File System Error', 'Could not open folder ' . $current_dir);
				}
			}
			return $file_list;
		}

		$root_dir = '../';
		$out = searchdir($root_dir);
		$pattern = "#get_(?:option|settings)[ ]?\([ ]?[\'\"]([-\w]+)[\'\"][ ]?\)#";
		$temp_file_options_arr = array();

		foreach ( $out as $found_file )
		{
			if ( !$handle = fopen($found_file, 'r') )
			{
				add_wp_error('File System Error', 'Could not open file ' . $found_file);
			}
			else
			{
				$fs = filesize($found_file);
				if ( $fs == 0 )
				{
					$fs = 1;
				}
				$file_data = fread($handle, $fs);
				preg_match_all($pattern, $file_data, $matches_arr);
				fclose($handle);
				clearstatcache();
				$temp_file_options_arr = array_unique ( array_merge ($temp_file_options_arr, $matches_arr[1]) );
			}			
		}

/* Search Options Table for Existing option values */
/* autoload = yes block (Plugins) */

		function get_all_yes_autoload_options()
		{
			global $wpdb;
			$all_yes_options = array();
			$yes_query = "SELECT option_name, option_value FROM $wpdb->options WHERE autoload LIKE '%yes%'";
			$yes_options = $wpdb->get_results($yes_query, ARRAY_A);

			if( !is_array($yes_options) )
			{
				add_wp_error('Database Error', $yes_query . ' (the Orphaned Options query) did not return an array');
				$yes_options = array();
			}

			foreach ( $yes_options as $yes_option )
			{
				if ( empty($yes_option['option_name']) )
				{
					add_wp_error('Empty Value', 'There is an autoload yes Option with No Name with the value: ' . wp_specialchars($yes_option['option_value']) );
				}
				else
				{
					$yes_value = maybe_unserialize($yes_option['option_value']);
					$all_yes_options[ $yes_option['option_name'] ] = apply_filters('pre_option_' . $yes_option['option_name'], $yes_value);
				}
			}

			return apply_filters('all_options', $all_yes_options);
		}

		$opt_arr = get_all_yes_autoload_options();
		$temp_table_options_arr = array();

		foreach ( $opt_arr as $key => $value )
		{
			$temp_table_options_arr[] =  $key;
		}

		echo "<p>The following Options appear to be orphans.<br />Non-selectable Options are known to have been created from files present during upgrade or backup, or are legitimate options that do not &#34;fit&#34; the search for get_option or get_settings. If you wish to remove them by other means, do so at your own risk.</p>";

		$orphans = array_diff($temp_table_options_arr, $temp_file_options_arr);
		unset($temp_table_options_arr, $temp_file_options_arr);
		natcasesort($orphans);
/*
* Options used by files in WordPress "core" should have been found
* and put into the $temp_file_options_arr array. But just in case,
* they're in the $known_ok array too. A bit of code bloat perhaps, but better
* than making it too easy to remove something that may break the blog.
* these options are found in wp-admin/includes/schema.php, and
* wp-content/plugins/akismet/akismet.php~, wp-content/themes/default/functions.php^,
* wp-cron.php#, wp-includes/cron.php@,
* wp-includes/rewrite.php!, wp-admin/options-reading.php*
*/
		$known_ok = array('active_plugins',			//2.7
				'admin_email',				//2.7
				'advanced_edit',			//2.7
				'akismet_discard_month',		//2.7~
				'akismet_spam_count',			//2.7~
				'blog_charset',				//2.7
				'blogdescription',			//2.7
				'blogname',				//2.7
				'category_base',			//2.7
				'close_comments_days_old',		//2.7
				'close_comments_for_old_posts',		//2.7
				'comment_max_links',			//2.7
				'comment_moderation',			//2.7
				'comment_order',			//2.7
				'comments_notify',			//2.7
				'comments_per_page',			//2.7
				'cron',					//2.7@
				'date_format',				//2.7
				'default_category',			//2.7
				'default_comment_status',		//2.7
				'default_comments_page',		//2.7
				'default_ping_status',			//2.7
				'default_pingback_flag',		//2.7
				'default_post_edit_rows',		//2.7
				'dismissed_update_core',		//2.7
				'doing_cron',				//2.7#
				'gmt_offset',				//2.7
				'gzipcompression',			//2.7
				'hack_file',				//2.7
				'home',					//2.7
				'image_default_align',			//2.7
				'image_default_link_type',		//2.7
				'image_default_size',			//2.7
				'kubrick_header_color',			//2.7^
				'kubrick_header_display',		//2.7^
				'kubrick_header_image',			//2.7^
				'large_size_h',				//2.7
				'large_size_w',				//2.7
				'links_recently_updated_append',	//2.7
				'links_recently_updated_prepend',	//2.7
				'links_recently_updated_time',		//2.7
				'links_updated_date_format',		//2.7
				'mailserver_login',			//2.7
				'mailserver_pass',			//2.7
				'mailserver_port',			//2.7
				'mailserver_url',			//2.7
				'moderation_keys',			//2.7
				'moderation_notify',			//2.7
				'page_comments',			//2.7
				'page_for_posts',			//2.7*
				'page_on_front',			//2.7*
				'permalink_structure',			//2.7
				'ping_sites',				//2.7
				'posts_per_page',			//2.7
				'posts_per_rss',			//2.7
				'require_name_email',			//2.7
				'rewrite_rules',			//2.7!
				'rss_excerpt_length',			//2.7
				'rss_use_excerpt',			//2.7
				'show_on_front',			//2.7*
				'siteurl',				//2.7
				'start_of_week',			//2.7
				'sticky_posts',				//2.7
				'thread_comments',			//2.7
				'thread_comments_depth',		//2.7
				'time_format',				//2.7
				'update_core',				//2.7
				'use_balanceTags',			//2.7
				'use_smilies',				//2.7
				'use_ssl',				//2.7
				'users_can_register',			//2.7
				'what_to_show',				//2.7
				'widget_akismet',			//2.7~
				'widget_categories',			//2.7
				'widget_rss',				//2.7
				'widget_text',				//2.7
				'wordpress_api_key',			//2.7~

				'avatar_default',			//2.6
				'enable_app',				//2.6
				'enable_xmlrpc',			//2.6
				'page_attachment_uris',			//2.6!

				'avatar_rating',			//2.5
				'medium_size_w',			//2.5
				'medium_size_h',			//2.5
				'show_avatars',				//2.5
				'thumbnail_crop',			//2.5
				'thumbnail_size_h',			//2.5
				'thumbnail_size_w',			//2.5
				'upload_url_path',			//2.5

				'xvalid_options',			//2.3 during backup

				'tag_base',				//2.2

				'import-blogger',			//2.1.3
				'blog_public',				//2.1
				'default_link_category',		//2.1
				'show_on_front',			//2.1

				'secret',				//2.0.3
				'upload_path',				//2.0.1
				'uploads_use_yearmonth_folders',	//2.0.1
				'db_version',				//2.0
				'default_role',				//2.0

				'use_trackback',			//1.5.1
				'blacklist_keys',			//1.5
				'comment_registration',			//1.5
				'comment_whitelist',			//1.5
				'default_email_category',		//1.5
				'html_type',				//1.5
				'page_uris',				//1.5
				'recently_edited',			//1.5
				'rss_language',				//1.5
				'stylesheet',				//1.5
				'template',				//1.5
				'use_linksupdate',			//1.5
				);

		if ( empty($orphans) )
		{
			echo "No Orphaned Options<br />";
		}
		else
		{
			foreach ( $orphans as $opt_val )
			{
/*
* the wp_user_roles option constructed from the capabilities class may not have
* the default install prefix. Test for it here while testing for the "known ok" array.
*/ 
				if ( ( in_array($opt_val, $known_ok) ) || ( strpos($opt_val, 'user_roles') !== FALSE ) )
				{
					echo "-&nbsp;&nbsp;" . $opt_val . "<br />";
				}
				else
				{
					echo "<input name='prune_opt[]' type='checkbox' value='" . $opt_val . "' />";
					echo "&nbsp;" . $opt_val . "<br />";
				}
			}
		}

/* autoload != yes block (RSS cache) */

		function get_all_no_autoload_options()
		{
			global $wpdb;
			$all_no_options = array();

			$no_query = "SELECT option_name, option_value FROM $wpdb->options WHERE autoload NOT LIKE '%yes%'";
			$no_options = $wpdb->get_results($no_query, ARRAY_A);

			if( !is_array($no_options) )
			{
				add_wp_error('Database Error', $no_query . ' (the "RSS_" Options query) did not return an array');
				$no_options = array();
			}

			foreach ( $no_options as $no_option )
			{
				if ( empty($no_option['option_name']) )
				{
					add_wp_error('Empty Value', 'There is an autoload not equal to yes Option with No Name with the value: ' . wp_specialchars($no_option['option_value']) );
				}
				else
				{
					$no_value = maybe_unserialize($no_option['option_value']);
					$all_no_options[ $no_option['option_name'] ] = apply_filters('pre_option_' . $no_option['option_name'], $no_value);
				}
			}
			return apply_filters('all_options', $all_no_options);
		}

		$temp_rss_opt_arr = get_all_no_autoload_options();

		check_for_wp_errors();

		function read_rss_ts($rss_opt_val)
		{
			$mtime = get_option($rss_opt_val);
			$age = time() - $mtime;
			$days = floor($age/86400);
			return $days;
		}

		$rss_opt_arr = array();
		$rss_ts_arr = array();
		$ts_regex = '/^(?:rss_)[a-f0-9]+(?:_ts)$/';
		$rss_regex = '/^(?:rss_)[a-f0-9]+$/';

		foreach ( $temp_rss_opt_arr as $key => $value )
		{
			if ( preg_match($ts_regex, $key) )
			{
				$rss_ts_arr[] = read_rss_ts($key);
				$rss_opt_arr[] =  $key;
			}
			else if ( preg_match($rss_regex, $key) )
			{
				$rss_opt_arr[] =  $key;
			}
		}
		unset($temp_rss_opt_arr);
		natcasesort($rss_ts_arr);
		$num_rss_days = count($rss_ts_arr);
		$ok_rss_date = '100';
		if ( $num_rss_days >= '15' )
		{
			$ok_rss_date = $rss_ts_arr['14'];
		}

		echo "<p>The following contains &#34;RSS&#34; Options added to the wp_options table from the blog's dashboard page and other files that parse RSS feeds and cache the results.<br />In each pair, the upper option is the cached feed and the lower is the option's timestamp.<br />Those listed may include options that are <b>Currently Active</b>.<br />When shown, rss_option pairs with dates newer or the same as the date of 7'th newest rss_ option pair (the ones that are more likely to be current) have no checkbox but begin with &#34;-&#34; and end with &#34;<em># days old</em>&#34; in italics.<br />The older rss_ options can be selected and end with &#34;<strong># days old</strong>&#34; in bold.</p>";

		natcasesort($rss_opt_arr);
		$rss_opt_arr = array_chunk($rss_opt_arr, 2, FALSE);

		if ( !empty($rss_opt_arr) && isset($_POST['limit_query']) && ($_POST['limit_query'] == '1') )
		{
			$rss_opt_arr = array_slice($rss_opt_arr, 0, (CO_BATCH_SIZE/2), true);
		}

		if ( empty($rss_opt_arr) )
		{
			echo "There are No &#34;RSS_&#34; Options<br />";
		}
		else
		{
			$odd_man_out = array();
			foreach ( $rss_opt_arr as $rss_opt_val )
			{
				$rss_opt_date = ( preg_match($ts_regex, $rss_opt_val[1]) ) ? read_rss_ts($rss_opt_val[1]) : $ok_rss_date;
				if ( strstr($rss_opt_val[1], $rss_opt_val[0]) && ( $rss_opt_date > $ok_rss_date ) )
				{
					if ( preg_match($rss_regex, $rss_opt_val[0]) )
					{
						echo "<input name='prune_opt[]' type='checkbox' value='" . $rss_opt_val[0] . "' />";
						echo "&nbsp;" . $rss_opt_val[0] . "&nbsp;&nbsp;&nbsp;<strong>" . $rss_opt_date . " days old</strong><br />";
					}
					if ( preg_match($ts_regex, $rss_opt_val[1]) )
					{
						echo "<input name='prune_opt[]' type='checkbox' value='" . $rss_opt_val[1] . "' />";
						echo "&nbsp;" . $rss_opt_val[1] . "&nbsp;&nbsp;&nbsp;<strong>" . $rss_opt_date . " days old</strong><br /><br />";
					}
				}
				else if ( strstr($rss_opt_val[1], $rss_opt_val[0]) && ( $rss_opt_date <= $ok_rss_date ) )
				{
					if ( preg_match($rss_regex, $rss_opt_val[0]) )
					{
						echo "-&nbsp;&nbsp;" . $rss_opt_val[0] . "&nbsp;&nbsp;&nbsp;<em>" . $rss_opt_date . " days old</em><br />";
					}
					if ( preg_match($ts_regex, $rss_opt_val[1]) )
					{
						echo "-&nbsp;&nbsp;" . $rss_opt_val[1] . "&nbsp;&nbsp;&nbsp;<em>" . $rss_opt_date . " days old</em><br /><br />";
					}
				}
				else
				{
					$rss_opt_arr_sz = count($rss_opt_val);
					for ($i = 0; $i < $rss_opt_arr_sz; $i++)
					{
						$odd_man_out[] = $rss_opt_val[$i];
					}
				}
				$rss_opt_date = '';
			}
			if ( !empty($odd_man_out) )
			{
				echo "<p><b>!WARNING!</b> The following options were not paired correctly. Be certain to check their information carefully before you remove them.</p>";
				$omo_arr_sz = count($odd_man_out);
				for ($j = 0; $j < $omo_arr_sz; $j++)
				{
					if ( preg_match($ts_regex, $odd_man_out[$j]) )
					{
						$rss_opt_date = read_rss_ts($odd_man_out[$j]);
						if ( $rss_opt_date > $ok_rss_date )
						{
							echo "<input name='prune_opt[]' type='checkbox' value='" . $odd_man_out[$j] . "' />";
							echo "&nbsp;" . $odd_man_out[$j] . "&nbsp;&nbsp;&nbsp;<strong>" . $rss_opt_date . " days old</strong><br />";
						}
						else if ( $rss_opt_date <= $ok_rss_date )
						{
							
							echo "-&nbsp;&nbsp;" . $odd_man_out[$j] . "&nbsp;&nbsp;&nbsp;<em>" . $rss_opt_date . " days old</em><br /><br />";
						}
					}
					else
					{
						echo "<input name='prune_opt[]' type='checkbox' value='" . $odd_man_out[$j] . "' />";
						echo "&nbsp;" . $odd_man_out[$j] . "&nbsp;&nbsp;&nbsp;<strong>!ALERT! Age Unknown</strong> check the age of the corresponding &#34;_ts&#34; option.<br />";
					}
				}
			}
		}

		if ( function_exists('wp_nonce_field') )
			wp_nonce_field('clean-options-pre-remove-orphans_' . $cononce);
?>
<!-- javascript select all modified from wp-db-backup 2.1.5 plugin by Austin Matzko http://www.ilfilosofo.com/blog/ -->
			<script type="text/javascript">
			//<![CDATA[
				var selectAllOptions = function() {};
				(function(b){
					var n = function(c) {
						var p = document.getElementsByTagName("input");
						for(var i=0; i<p.length; i++)
							if( 'prune_opt[]' == p[i].getAttribute('name') ) p[i].checked = c;
					}
					b.a = function() { n(true) }
					b.n = function() { n(false) }
					document.write('<p><a href="javascript:void(0)" onclick="selectAllOptions.a()">Select all</a> / <a href="javascript:void(0)" onclick="selectAllOptions.n()">Deselect all</a></p>');
				})(selectAllOptions)
			//]]>
			</script>

		<div class="submit">
			<input type="submit" name="pre_remove_orphans" value="View Selected Options Information" />
		</div>
		</form>
		</div>
<?php
	}

/* Pre Remove Orphans Section */

	else if ( isset($_POST['pre_remove_orphans']) )
	{
		check_admin_referer('clean-options-pre-remove-orphans_' . $cononce);
?>
		<div class="wrap">
		<h2>Clean Options</h2>
		<p>*Note* spaces have been added after every 10th character of the option_name and every 20th character of the option_value to preserve page layout.<br />Not all options have values and / or descriptions.</p><p><strong>Please review this information very carefully and only remove Options that you know for certain have been orphaned or deprecated.</strong></p><p><strong>It is strongly suggested that you BACKUP your database before removing any options.</strong><p>
<?php
		if ( ( isset($_POST['prune_opt']) ) && ( is_array($_POST['prune_opt']) ) )
		{
			global $wpdb;

			$prune_opt_arr = $_POST['prune_opt'];
			if ( $cur_wp_ver < '2.3' )
			{
				$prune_query = "SELECT option_name, option_value, option_description FROM $wpdb->options WHERE ";
			}
			else if ( $cur_wp_ver > '2.2' )
			{
				$prune_query = "SELECT option_name, option_value FROM $wpdb->options WHERE ";
			}
			$prune_wheres = '';
			foreach ( $prune_opt_arr as $pruneopt )
			{
				if ( !empty($pruneopt) )
				{
					$prune_wheres .= "option_name LIKE '" . $pruneopt . "' OR ";
				}
			}
			$prune_wheres = rtrim($prune_wheres, ' OR ');
			$prune_query .= $prune_wheres;
			$q_results = $wpdb->get_results($prune_query, ARRAY_A);
			echo "<table id='or_table'>";
			if ( $cur_wp_ver < '2.3' )
			{
				echo "<tr><th>option_name</th><th>option_value</th><th>option_description</th></tr>";
			}
			else if ( $cur_wp_ver > '2.2' )
			{
				echo "<tr><th>option_name</th><th>option_value</th></tr>";
			}

			if( !is_array($q_results) )
			{
				add_wp_error('Database Error', $prune_query . ' (the review information query) did not return an array');
				$q_results = array();
			}

			foreach ( $q_results as $q_val )
			{
				$opt_name = $q_val['option_name'];
				$opt_name = wordwrap($opt_name, 10, ' ', 1);
				echo "<tr><td>" . $opt_name . "</td>";
				$opt_val = htmlentities( print_r($q_val['option_value'], TRUE) );
				$opt_val = wordwrap($opt_val, 20, ' ', 1);
				echo "<td>" . $opt_val . "</td>";

				if ( $cur_wp_ver < '2.3' )
				{
					$opt_desc = print_r($q_val['option_description'], TRUE);
					echo "<td>" . $opt_desc . "</td></tr>";
				}
			}
			echo "</table>";
		}

		check_for_wp_errors();

		$opt_arr_str = ( empty($prune_opt_arr) ) ? '' : implode('#', $prune_opt_arr);
		if ( !empty($opt_arr_str) )
		{

?>
			<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
			if ( function_exists('wp_nonce_field') )
				wp_nonce_field('clean-options-do-remove-orphans_' . $cononce);
?>
			<input type="hidden" name="orphan_opts" value="<?php echo $opt_arr_str; ?>" />
			<input type="radio" name="confirm_rem" value="1" /> Yes, Remove ALL of these options from the wp_options table.<br />
			<input type="radio" name="confirm_rem" value="0" checked="checked" /> No, Don't remove them, return to the first screen.<br />
			<div class="submit">
				<input type="submit" name="do_remove_orphans" value="Submit" />
			</div>
			</form>
<?php
		}
		else
		{
			echo "<b>No Orphaned Options where selected</b><br />";
			unset($_POST);
			echo "Return to the <a href='" . $_SERVER["REQUEST_URI"] . "'>first screen</a><br />";			
		}
?>
		</div>
<?php
	}

/* Do Remove Orphans Section */

	else if ( isset($_POST['do_remove_orphans']) )
	{
		check_admin_referer('clean-options-do-remove-orphans_' . $cononce);

		if ( ( isset($_POST['confirm_rem']) ) && ($_POST['confirm_rem'] == "1") )
		{
			if ( !empty($_POST['orphan_opts']) )
			{
?>
				<div id="message" class="updated fade">
				<p><strong>Removed Options:</strong><br />
<?php
				$orphan_opt_arr = explode("#", $_POST['orphan_opts']);
				foreach ( $orphan_opt_arr as $orphanopt )
				{
					delete_option($orphanopt);
					echo $orphanopt . "<br />";
				}
?>
				</p>
				</div>
<?php
			}
		}
?>
		<div class="wrap">
		<h2>Clean Options</h2>
<?php
		echo $common_form_message;
		$opt_tbl_len = get_opt_tbl_len();
?>
		<p>The Options table currently has <b><?php echo $opt_tbl_len; ?></b> rows.</p>
<?php
		if ( $opt_tbl_len > CO_MIN_BEFORE_SHOW )
		{
			echo '<p>Most likely there are an extreme number of "rss_hash" rows in the wp_options table.</p>';
			echo $del_all_warning;
?>
		<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
			if ( function_exists('wp_nonce_field') )
				wp_nonce_field('clean-options-del-all-rss_' . $cononce);
?>
			<div class="submit">
				<input type="submit" name="del_all_rss" value="Delete ALL 'rss' Options" />
			</div>
		</form>
<?php
		}
?>
		<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
		if ( function_exists('wp_nonce_field') )
			wp_nonce_field('clean-options-find-orphans_' . $cononce);

		if ( $opt_tbl_len > CO_BATCH_SIZE )
		{
			echo "<input name='limit_query' id='limit_query' type='checkbox' value='1' checked='checked' />";
			echo "&nbsp;<label for='limit_query'>Limit the number of rss_hash options found to only the " . CO_BATCH_SIZE . " oldest.</label><br />";
		}
?>
		<div class="submit">
			<input type="submit" name="find_orphans" value="Find Orphaned Options" />
		</div>
		</form>
		</div>
<?php
	}

/* Delete All RSS Section */

	else if ( isset($_POST['del_all_rss']) )
	{
		check_admin_referer('clean-options-del-all-rss_' . $cononce);

		global $wpdb;

		$last_option_id = $wpdb->get_results("SELECT option_id FROM $wpdb->options ORDER BY option_id DESC LIMIT 1", ARRAY_A);
		$maybe_safe = $last_option_id[0]['option_id'] - CO_KEEP_NEWER;
		$wpdb->query("DELETE FROM $wpdb->options WHERE option_name LIKE 'rss_%' AND option_id < $maybe_safe");

		$rows_deleted = $wpdb->rows_affected;
?>
		<div id="message" class="updated fade">
		<p><strong>Removed <?php echo $rows_deleted; ?> "rss_hash" Options</strong></p>
		</div>
		<div class="wrap">
		<h2>Clean Options</h2>
<?php
		echo $common_form_message;
?>
		<p>The Options table currently has <b><?php echo get_opt_tbl_len(); ?></b> rows.</p>
		<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
		if ( function_exists('wp_nonce_field') )
			wp_nonce_field('clean-options-find-orphans_' . $cononce);
?>
		<p>Every "rss_hash" option in the wp_options table will be shown, including current ones.</p>
		<div class="submit">
			<input type="submit" name="find_orphans" value="Find Orphaned Options" />
		</div>
		</form>
		</div>
<?php
	}

/* Intitial View Section */

	else
	{
?>
		<div class="wrap">
		<h2>Clean Options</h2>
<?php
		echo $common_form_message;
		$opt_tbl_len = get_opt_tbl_len();
?>
		<p>The Options table currently has <b><?php echo $opt_tbl_len; ?></b> rows.</p>
<?php
		if ( $opt_tbl_len > CO_MIN_BEFORE_SHOW )
		{
			echo '<p>Most likely there are an extreme number of "rss_hash" rows in the wp_options table.</p>';
			echo $del_all_warning;
?>
		<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
			if ( function_exists('wp_nonce_field') )
				wp_nonce_field('clean-options-del-all-rss_' . $cononce);
?>
			<div class="submit">
				<input type="submit" name="del_all_rss" value="Delete ALL 'rss' Options" />
			</div>
		</form>
<?php
		}
?>
		<form method="post" action="<?php echo $_SERVER["REQUEST_URI"]; ?>">
<?php
		if ( function_exists('wp_nonce_field') )
			wp_nonce_field('clean-options-find-orphans_' . $cononce);

		if ( $opt_tbl_len > CO_BATCH_SIZE )
		{
			echo "<input name='limit_query' id='limit_query' type='checkbox' value='1' checked='checked' />";
			echo "&nbsp;<label for='limit_query'>Limit the number of rss_hash options found to only the " . CO_BATCH_SIZE . " oldest.</label><br />";
		}
?>
		<div class="submit">
			<input type="submit" name="find_orphans" value="Find Orphaned Options" />
		</div>
		</form>
		</div>
<?php
	}
?>

<!-- /* FURTHER INFORMATION SECTION */ -->

	<div class="wrap">
	<h2>Further Information</h2>
	<p>WANTED - Bug Reports<br />
	If you find any problems please let me know.</p>
	<p>For more information, the latest version, etc. please visit <a href='http://www.mittineague.com/dev/co.php'>http://www.mittineague.com/dev/co.php</a></p>
	<p>Questions? For support, please visit <a href='http://www.mittineague.com/forums/viewtopic.php?t=101'>http://www.mittineague.com/forums/viewtopic.php?t=101</a> (registration required to post)</p>
	<p>For comments / suggestions, please visit <a href='http://www.mittineague.com/blog/2007/04/clean-options-plugin/'>http://www.mittineague.com/blog/2007/04/clean-options-plugin/</a></p>
	</div>
<?php
}

if ( function_exists('add_action') )
{
	add_action('admin_head', 'mitt_co_css');
	add_action('admin_menu', 'mitt_add_co_page');
}
?>